@@ -29,7 +29,6 @@ from wimpiggy.lowlevel import (
                geometry_with_border,                        #@UnresolvedImport
                calc_constrained_size,                       #@UnresolvedImport
                is_mapped,                                   #@UnresolvedImport
-               show_unraised_without_extra_stupid_stuff,    #@UnresolvedImport
                unmap_with_serial,                           #@UnresolvedImport
                XDeleteProperty,                             #@UnresolvedImport
                XAddToSaveSet,                               #@UnresolvedImport
@@ -52,6 +51,10 @@ from wimpiggy.composite import CompositeHelper
 from wimpiggy.log import Logger
 log = Logger()
 
+if gtk.pygtk_version<(2,17):
+    log.error("your version of PyGTK is too old - expect some bugs")
+
+
 # Todo:
 #   client focus hints
 #   _NET_WM_SYNC_REQUEST
@@ -253,15 +256,15 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
             # Keith Packard says that composite state is undefined following a
             # reparent, so I'm not sure doing this here in the superclass,
             # before we reparent, actually works... let's wait and see.
-            self._composite = CompositeHelper(self.client_window, False)
-            h = self._composite.connect("contents-changed",
-                                        self._forward_contents_changed)
+            self._composite = CompositeHelper(self.client_window, False, )  #self.composite_configure_event)
+            h = self._composite.connect("contents-changed", self._forward_contents_changed)
+            self._composite.connect("wimpiggy-configure-event", self.composite_configure_event)
             self._damage_forward_handle = h
             self._geometry = geometry_with_border(self.client_window)
         try:
             trap.call(setup)
         except XError, e:
-            raise Unmanageable, e
+            raise Unmanageable(e)
 
     def _forward_contents_changed(self, obj, event):
         self.emit("client-contents-changed", event)
@@ -280,6 +283,9 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
                           event.border_width)
         self.notify("geometry")
 
+    def composite_configure_event(self, composite_window, event):
+        log("BaseWindowModel.composite_configure_event(%s,%s)", composite_window, event)
+
     def do_get_property_geometry(self, pspec):
         (x, y, w, h, b) = self._geometry
         return (x, y, w + 2*b, h + 2*b)
@@ -299,18 +305,19 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         self._internal_set_property("transient-for", transient_for)
 
         window_types = self.prop_get("_NET_WM_WINDOW_TYPE", ["atom"])
-        if window_types:
-            self._internal_set_property("window-type", window_types)
-        else:
-            if transient_for is not None:
-                # EWMH says that even if it's transient-for, we MUST check to
-                # see if it's override-redirect (and if so treat as NORMAL).
-                # But we wouldn't be here if this was override-redirect.
-                assume_type = "_NET_WM_TYPE_DIALOG"
-            else:
-                assume_type = "_NET_WM_WINDOW_TYPE_NORMAL"
-            self._internal_set_property("window-type",
-                              [gtk.gdk.atom_intern(assume_type)])
+        if not window_types:
+            window_type = self._guess_window_type(transient_for)
+            window_types = [gtk.gdk.atom_intern(window_type)]
+        self._internal_set_property("window-type", window_types)
+
+    def _guess_window_type(self, transient_for):
+        if transient_for is not None:
+            # EWMH says that even if it's transient-for, we MUST check to
+            # see if it's override-redirect (and if so treat as NORMAL).
+            # But we wouldn't be here if this was override-redirect.
+            # (OverrideRedirectWindowModel overrides this method)
+            return "_NET_WM_TYPE_DIALOG"
+        return "_NET_WM_WINDOW_TYPE_NORMAL"
 
 
 gobject.type_register(BaseWindowModel)
@@ -334,13 +341,16 @@ class OverrideRedirectWindowModel(BaseWindowModel):
             # already generated, and our request for that event is too late!
             # So double check now, *after* putting in our request:
             if not is_mapped(self.client_window):
-                raise Unmanageable, "window already unmapped"
+                raise Unmanageable("window already unmapped")
 
             self._read_initial_properties()
         try:
             trap.call(setup)
         except XError, e:
-            raise Unmanageable, e
+            raise Unmanageable(e)
+
+    def _guess_window_type(self, transient_for):
+        return "_NET_WM_WINDOW_TYPE_NORMAL"
 
     def do_wimpiggy_unmap_event(self, event):
         self.unmanage()
@@ -441,6 +451,7 @@ class WindowModel(BaseWindowModel):
     __gsignals__ = {
         # X11 bell event:
         "bell": one_arg_signal,
+        "geometry": one_arg_signal,
         
         "ownership-election": (gobject.SIGNAL_RUN_LAST,
                                gobject.TYPE_PYOBJECT, (),
@@ -507,14 +518,20 @@ class WindowModel(BaseWindowModel):
 
             XAddToSaveSet(self.client_window)
             self.client_window.reparent(self.corral_window, 0, 0)
-            client_size = self.client_window.get_geometry()[2:4]
+            w,h = self.client_window.get_geometry()[2:4]
+            hints = self.get_property("size-hints")
+            self._sanitize_size_hints(hints)
+            client_size = calc_constrained_size(w, h, hints)[:2]
+            log("setup_client() resizing windows to %s", client_size)
+            self.client_window.resize(*client_size)
             self.corral_window.resize(*client_size)
-            # We CANNOT use .show_unraised here, because of GTK+ bug #526635:
-            show_unraised_without_extra_stupid_stuff(self.client_window)
+            self.client_window.show_unraised()
+            self.client_window.get_geometry()
         try:
             trap.call(setup_client)
+            trap.call(self.client_window.get_geometry)
         except XError, e:
-            raise Unmanageable, e
+            raise Unmanageable(e)
         self._setup_done = True
 
     def prop_get(self, key, type, ignore_errors=False):
@@ -590,9 +607,7 @@ class WindowModel(BaseWindowModel):
             XRemoveFromSaveSet(self.client_window)
             sendConfigureNotify(self.client_window)
             if exiting:
-                # We CANNOT use .show_unraised here, because of GTK+ bug
-                # #526635:
-                show_unraised_without_extra_stupid_stuff(self.client_window)
+                self.client_window.show_unraised()
         trap.swallow(unmanageit)
         self.corral_window.destroy()
         BaseWindowModel.do_unmanaged(self, exiting)
@@ -638,20 +653,51 @@ class WindowModel(BaseWindowModel):
     def _update_client_geometry(self):
         owner = self.get_property("owner")
         if owner is not None:
-            (allocated_w, allocated_h) = owner.window_size(self)
+            log("_update_client_geometry: owner()=%s", owner)
+            def window_size():
+                return  owner.window_size(self)
+            def window_position(w, h):
+                return  owner.window_position(self, w, h)
+            self._do_update_client_geometry(window_size, window_position)
+
+    def _do_update_client_geometry(self, window_size_cb, window_position_cb):
+        allocated_w, allocated_h = window_size_cb()
+        log("_do_update_client_geometry: %sx%s", allocated_w, allocated_h)
+        hints = self.get_property("size-hints")
+        self._sanitize_size_hints(hints)
+        size = calc_constrained_size(allocated_w, allocated_h, hints)
+        log("_do_update_client_geometry: size=%s", size)
+        w, h, wvis, hvis = size
+        x, y = window_position_cb(w, h)
+        log("_do_update_client_geometry: position=%s", (x,y))
+        self.corral_window.move_resize(x, y, w, h)
+        trap.swallow(configureAndNotify, self.client_window, 0, 0, w, h)
+        self._internal_set_property("actual-size", (w, h))
+        self._internal_set_property("user-friendly-size", (wvis, hvis))
+
+    def composite_configure_event(self, composite_window, event):
+        #the client window may have been resized (generally programmatically)
+        #so we may need to update the corral_window to match
+        log("WindowModel.composite_configure_event(%s, %s)", composite_window, event)
+        cow, coh = self.corral_window.get_geometry()[2:4]
+        clw, clh = self.client_window.get_geometry()[2:4]
+        if cow!=clw or coh!=clh:
+            log("WindowModel.composite_configure_event(%s, %s) corral window (%sx%s) does not match client window (%sx%s), resizing it",
+                     composite_window, event, cow, coh, clw, clh)
+            self.corral_window.resize(clw, clh)
             hints = self.get_property("size-hints")
             self._sanitize_size_hints(hints)
-            size = calc_constrained_size(allocated_w, allocated_h, hints)
-            (w, h, wvis, hvis) = size
-            (x, y) = owner.window_position(self, w, h)
-            self.corral_window.move_resize(x, y, w, h)
-            trap.swallow(configureAndNotify, self.client_window, 0, 0, w, h)
+            size = calc_constrained_size(clw, clh, hints)
+            log("composite_configure_event: new constrained size=%s", size)
+            w, h, wvis, hvis = size
             self._internal_set_property("actual-size", (w, h))
             self._internal_set_property("user-friendly-size", (wvis, hvis))
+            self.emit("geometry", event)
 
     def do_child_configure_request_event(self, event):
         # Ignore the request, but as per ICCCM 4.1.5, send back a synthetic
         # ConfigureNotify telling the client that nothing has happened.
+        log("do_child_configure_request_event(%s)", event)
         trap.swallow(sendConfigureNotify, event.window)
 
         # Also potentially update our record of what the app has requested:
@@ -715,7 +761,7 @@ class WindowModel(BaseWindowModel):
         # their properties every time they see a ConfigureNotify, and this
         # reduces the chance for us to get caught in loops:
         old_hints = self.get_property("size-hints")
-        if old_hints is None or size_hints.__dict__ != old_hints.__dict__:
+        if size_hints and (old_hints is None or size_hints.__dict__ != old_hints.__dict__):
             self._internal_set_property("size-hints", size_hints)
             self._update_client_geometry()
 
@@ -845,9 +891,13 @@ class WindowModel(BaseWindowModel):
         for mutable in ["WM_HINTS", "WM_NORMAL_HINTS",
                         "WM_NAME", "_NET_WM_NAME",
                         "WM_ICON_NAME", "_NET_WM_ICON_NAME",
-                        "_NET_WM_STRUT", "_NET_WM_STRUT_PARTIAL",
-                        "_NET_WM_ICON"]:
+                        "_NET_WM_STRUT", "_NET_WM_STRUT_PARTIAL"]:
             self._handle_property_change(mutable)
+        for mutable in ["_NET_WM_ICON"]:
+            try:
+                self._handle_property_change(mutable)
+            except:
+                log.error("error reading initial property %s", mutable, exc_info=True)
 
     ################################
     # Property setting
@@ -952,10 +1002,10 @@ class WindowModel(BaseWindowModel):
                   "_NET_FRAME_EXTENTS",
                   "_NET_WM_ALLOWED_ACTIONS",
                   ]
-        def doit():
+        def delete_properties():
             for prop in remove:
                 XDeleteProperty(self.client_window, prop)
-        trap.swallow(doit)
+        trap.swallow(delete_properties)
 
     ################################
     # Focus handling:
@@ -1216,21 +1266,22 @@ class WindowView(gtk.Widget):
         # widgets.  For now the assumption is that we're pretty much always
         # going to get a size imposed on us, so no point in spending excessive
         # effort figuring out what requisition means in this context.
-        (requisition.width, requisition.height) = (100, 100)
+        log("do_size_request(%s)", requisition)
+        requisition.width, requisition.height = 100, 100
 
     def do_size_allocate(self, allocation):
         self.allocation = allocation
-        log("New allocation = %r", tuple(self.allocation))
+        log("do_size_allocate(%r)", tuple(self.allocation))
         if self.flags() & gtk.REALIZED:
             self.window.move_resize(*allocation)
             self._image_window.resize(allocation.width, allocation.height)
-            self._image_window.input_shape_combine_region(gtk.gdk.Region(),
-                                                          0, 0)
+            self._image_window.input_shape_combine_region(gtk.gdk.Region(), 0, 0)
         self.model.ownership_election()
         self.model.maybe_recalculate_geometry_for(self)
 
     def window_size(self, model):
         assert self.flags() & gtk.REALIZED
+        log("window_size(%s)=%s", model, (self.allocation.width, self.allocation.height))
         return (self.allocation.width, self.allocation.height)
 
     def take_window(self, model, window):
