@@ -11,83 +11,14 @@ gobject = import_gobject()
 gtk = import_gtk()
 gdk = import_gdk()
 if is_gtk3():
-    def get_modifiers_mask():
-        return gdk.get_default_root_window().get_pointer()[-1]
     def get_root_size():
         return gdk.get_default_root_window().get_geometry()[2:]
-    def init_window(win, wintype):
-        #TODO: no idea how to do this with gtk3
-        #maybe not even possible..
-        gtk.Window.__init__(win)
-    def get_window_geometry(gtkwindow):
-        x, y = gtkwindow.get_position()
-        w, h = gtkwindow.get_size()
-        return (x, y, w, h)
-    def set_geometry_hints(window, hints):
-        """ we convert the hints as a dict into a gdk.Geometry + gdk.WindowHints """
-        wh = gdk.WindowHints
-        name_to_hint = {"maximum-size"  : wh.MAX_SIZE,
-                        "max_width"     : wh.MAX_SIZE,
-                        "max_height"    : wh.MAX_SIZE,
-                        "minimum-size"  : wh.MIN_SIZE,
-                        "min_width"     : wh.MIN_SIZE,
-                        "min_height"    : wh.MIN_SIZE,
-                        "base-size"     : wh.BASE_SIZE,
-                        "base_width"    : wh.BASE_SIZE,
-                        "base_height"   : wh.BASE_SIZE,
-                        "increment"     : wh.RESIZE_INC,
-                        "width_inc"     : wh.RESIZE_INC,
-                        "height_inc"    : wh.RESIZE_INC,
-                        "min_aspect_ratio"  : wh.ASPECT,
-                        "max_aspect_ratio"  : wh.ASPECT,
-                        }
-        #these fields can be copied directly to the gdk.Geometry as ints:
-        INT_FIELDS= ["min_width",    "min_height",
-                        "max_width",    "max_height",
-                        "base_width",   "base_height",
-                        "width_inc",    "height_inc"]
-        ASPECT_FIELDS = {
-                        "min_aspect_ratio"  : "min_aspect",
-                        "max_aspect_ratio"  : "max_aspect",
-                         }
-        geom = gdk.Geometry()
-        mask = 0
-        for k,v in hints:
-            if k in INT_FIELDS:
-                setattr(geom, k, int(v))
-                mask |= int(name_to_hint.get(k, 0))
-            elif k in ASPECT_FIELDS:
-                field = ASPECT_FIELDS.get(k)
-                setattr(geom, field, float(v))
-                mask |= int(name_to_hint.get(k, 0))
-        hints = gdk.WindowHints(mask)
-        window.set_geometry_hints(None, geom, hints)
-
     def set_windows_cursor(gtkwindows, new_cursor):
         pass
         #window.override_cursor(cursor, None)
-    def queue_draw(window, x, y, width, height):
-        window.queue_draw_area(x, y, width, height)
-    WINDOW_POPUP = gtk.WindowType.POPUP
-    WINDOW_TOPLEVEL = gtk.WindowType.TOPLEVEL
-    WINDOW_EVENT_MASK = 0
-    OR_TYPE_HINTS = []
-    NAME_TO_HINT = { }
-    SCROLL_MAP = {}
 else:
-    def get_modifiers_mask():
-        return  gdk.get_default_root_window().get_pointer()[-1]
     def get_root_size():
         return gdk.get_default_root_window().get_size()
-    def init_window(gtkwindow, wintype):
-        gtk.Window.__init__(gtkwindow, wintype)
-    def get_window_geometry(gtkwindow):
-        gdkwindow = gtkwindow.get_window()
-        x, y = gdkwindow.get_origin()
-        _, _, w, h, _ = gdkwindow.get_geometry()
-        return (x, y, w, h)
-    def set_geometry_hints(gtkwindow, hints):
-        gtkwindow.set_geometry_hints(None, **hints)
     def set_windows_cursor(gtkwindows, new_cursor):
         cursor = None
         if len(new_cursor)>0:
@@ -106,49 +37,9 @@ else:
         for gtkwindow in gtkwindows:
             gtkwindow.get_window().set_cursor(cursor)
 
-    def queue_draw(gtkwindow, x, y, width, height):
-        gtkwindow.get_window().invalidate_rect(gdk.Rectangle(x, y, width, height), False)
-    WINDOW_POPUP = gtk.WINDOW_POPUP
-    WINDOW_TOPLEVEL = gtk.WINDOW_TOPLEVEL
-    WINDOW_EVENT_MASK = gdk.STRUCTURE_MASK | gdk.KEY_PRESS_MASK | gdk.KEY_RELEASE_MASK | gdk.POINTER_MOTION_MASK | gdk.BUTTON_PRESS_MASK | gdk.BUTTON_RELEASE_MASK
-    OR_TYPE_HINTS = [gdk.WINDOW_TYPE_HINT_DIALOG,
-                gdk.WINDOW_TYPE_HINT_MENU, gdk.WINDOW_TYPE_HINT_TOOLBAR,
-                #gdk.WINDOW_TYPE_HINT_SPLASHSCREEN, gdk.WINDOW_TYPE_HINT_UTILITY,
-                #gdk.WINDOW_TYPE_HINT_DOCK, gdk.WINDOW_TYPE_HINT_DESKTOP,
-                gdk.WINDOW_TYPE_HINT_DROPDOWN_MENU, gdk.WINDOW_TYPE_HINT_POPUP_MENU,
-                gdk.WINDOW_TYPE_HINT_TOOLTIP,
-                #gdk.WINDOW_TYPE_HINT_NOTIFICATION,
-                gdk.WINDOW_TYPE_HINT_COMBO,gdk.WINDOW_TYPE_HINT_DND]
-    NAME_TO_HINT = {
-                "_NET_WM_WINDOW_TYPE_NORMAL"    : gdk.WINDOW_TYPE_HINT_NORMAL,
-                "_NET_WM_WINDOW_TYPE_DIALOG"    : gdk.WINDOW_TYPE_HINT_DIALOG,
-                "_NET_WM_WINDOW_TYPE_MENU"      : gdk.WINDOW_TYPE_HINT_MENU,
-                "_NET_WM_WINDOW_TYPE_TOOLBAR"   : gdk.WINDOW_TYPE_HINT_TOOLBAR,
-                "_NET_WM_WINDOW_TYPE_SPLASH"    : gdk.WINDOW_TYPE_HINT_SPLASHSCREEN,
-                "_NET_WM_WINDOW_TYPE_UTILITY"   : gdk.WINDOW_TYPE_HINT_UTILITY,
-                "_NET_WM_WINDOW_TYPE_DOCK"      : gdk.WINDOW_TYPE_HINT_DOCK,
-                "_NET_WM_WINDOW_TYPE_DESKTOP"   : gdk.WINDOW_TYPE_HINT_DESKTOP,
-                "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU" : gdk.WINDOW_TYPE_HINT_DROPDOWN_MENU,
-                "_NET_WM_WINDOW_TYPE_POPUP_MENU": gdk.WINDOW_TYPE_HINT_POPUP_MENU,
-                "_NET_WM_WINDOW_TYPE_TOOLTIP"   : gdk.WINDOW_TYPE_HINT_TOOLTIP,
-                "_NET_WM_WINDOW_TYPE_NOTIFICATION" : gdk.WINDOW_TYPE_HINT_NOTIFICATION,
-                "_NET_WM_WINDOW_TYPE_COMBO"     : gdk.WINDOW_TYPE_HINT_COMBO,
-                "_NET_WM_WINDOW_TYPE_DND"       : gdk.WINDOW_TYPE_HINT_DND
-                }
-        # Map scroll directions back to mouse buttons.  Mapping is taken from
-        # gdk/x11/gdkevents-x11.c.
-    SCROLL_MAP = {gdk.SCROLL_UP: 4,
-                  gdk.SCROLL_DOWN: 5,
-                  gdk.SCROLL_LEFT: 6,
-                  gdk.SCROLL_RIGHT: 7,
-                  }
 
-
-import cairo
-import re
 import os
 import time
-import sys
 import ctypes
 
 from wimpiggy.util import (n_arg_signal,
@@ -161,302 +52,31 @@ log = Logger()
 from xpra.deque import maxdeque
 from xpra.client_base import XpraClientBase
 from xpra.keys import mask_to_names, DEFAULT_MODIFIER_MEANINGS, DEFAULT_MODIFIER_NUISANCE, DEFAULT_MODIFIER_IGNORE_KEYNAMES
-from xpra.window_backing import new_backing
 from xpra.platform.gui import ClientExtras
 from xpra.scripts.main import ENCODINGS
 
-if sys.version < '3':
-    import codecs
-    def u(x):
-        return codecs.unicode_escape_decode(x)[0]
-else:
-    def u(x):
-        return x
+from xpra.client_window import ClientWindow
+ClientWindowClass = ClientWindow
+#the GL backend only works with gtk2
+USE_OPENGL = False
+if USE_OPENGL and not is_gtk3():
+    try:
+        from xpra.gl_client_window import GLClientWindow
+        ClientWindowClass = GLClientWindow
+    except ImportError, e:
+        log.info("Disabled OpenGL output: %s" % e)
+
 def nn(x):
     if x is None:
         return  ""
     return x
 
-class ClientWindow(gtk.Window):
-    def __init__(self, client, wid, x, y, w, h, metadata, override_redirect):
-        if override_redirect:
-            init_window(self, WINDOW_POPUP)
-        else:
-            init_window(self, WINDOW_TOPLEVEL)
-        self._client = client
-        self._id = wid
-        self._pos = (-1, -1)
-        self._size = (1, 1)
-        self._backing = None
-        self.new_backing(w, h)
-        self._metadata = {}
-        self._override_redirect = override_redirect
-        self._refresh_timer = None
-        self._refresh_requested = False
-        # used for only sending focus events *after* the window is mapped:
-        self._been_mapped = False
-        self._override_redirect_windows = []
-        # tell KDE/oxygen not to intercept clicks
-        # see: https://bugs.kde.org/show_bug.cgi?id=274485
-        self.set_data("_kde_no_window_grab", 1)
-
-        self.update_metadata(metadata)
-
-        self.set_app_paintable(True)
-        self.add_events(WINDOW_EVENT_MASK)
-        self.move(x, y)
-        self.set_default_size(w, h)
-        if override_redirect:
-            transient_for = self.get_transient_for()
-            type_hint = self.get_type_hint()
-            if transient_for is not None and transient_for.window is not None and type_hint in OR_TYPE_HINTS:
-                transient_for._override_redirect_windows.append(self)
-        self.connect("notify::has-toplevel-focus", self._focus_change)
-
-    def new_backing(self, w, h):
-        self._backing = new_backing(self._id, w, h, self._backing, self._client.supports_mmap, self._client.mmap)
-
-    def update_metadata(self, metadata):
-        self._metadata.update(metadata)
-
-        title = u(self._client.title)
-        if title.find("@")>=0:
-            #perform metadata variable substitutions:
-            default_values = {"title" : u("<untitled window>"),
-                              "client-machine" : u("<unknown machine>")}
-            def metadata_replace(match):
-                atvar = match.group(0)          #ie: '@title@'
-                var = atvar[1:len(atvar)-1]     #ie: 'title'
-                default_value = default_values.get(var, u("<unknown %s>") % var)
-                value = self._metadata.get(var, default_value)
-                return value.decode("utf-8")
-            title = re.sub("@[\w\-]*@", metadata_replace, title)
-        self.set_title(title)
-
-        if "size-constraints" in self._metadata:
-            size_metadata = self._metadata["size-constraints"]
-            hints = {}
-            for (a, h1, h2) in [
-                ("maximum-size", "max_width", "max_height"),
-                ("minimum-size", "min_width", "min_height"),
-                ("base-size", "base_width", "base_height"),
-                ("increment", "width_inc", "height_inc"),
-                ]:
-                if a in self._metadata["size-constraints"]:
-                    hints[h1], hints[h2] = size_metadata[a]
-            for (a, h) in [
-                ("minimum-aspect", "min_aspect_ratio"),
-                ("maximum-aspect", "max_aspect_ratio"),
-                ]:
-                if a in self._metadata:
-                    hints[h] = size_metadata[a][0] * 1.0 / size_metadata[a][1]
-            set_geometry_hints(self, hints)
-
-        if hasattr(self, "get_realized"):
-            #pygtk 2.22 and above have this method:
-            realized = self.get_realized()
-        else:
-            #older versions:
-            realized = self.flags() & gtk.REALIZED
-        if not realized:
-            self.set_wmclass(*self._metadata.get("class-instance",
-                                                 ("xpra", "Xpra")))
-
-        if "icon" in self._metadata:
-            (width, height, coding, data) = self._metadata["icon"]
-            if coding == "premult_argb32":
-                cairo_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
-                cairo_surf.get_data()[:] = data
-                # FIXME: We round-trip through PNG. This is ridiculous, but faster
-                # than doing a bunch of alpha un-premultiplying and byte-swapping
-                # by hand in Python (better still would be to write some Pyrex,
-                # but I don't have time right now):
-                loader = gdk.PixbufLoader()
-                cairo_surf.write_to_png(loader)
-                loader.close()
-                pixbuf = loader.get_pixbuf()
-            else:
-                loader = gdk.PixbufLoader(coding)
-                loader.write(data, len(data))
-                loader.close()
-                pixbuf = loader.get_pixbuf()
-            self.set_icon(pixbuf)
-
-        if "transient-for" in self._metadata:
-            wid = self._metadata.get("transient-for")
-            window = self._client._id_to_window.get(wid)
-            log.debug("found transient-for: %s / %s", wid, window)
-            if window:
-                self.set_transient_for(window)
-
-        if "window-type" in self._metadata:
-            window_types = self._metadata.get("window-type")
-            log.debug("window types=%s", window_types)
-            for window_type in window_types:
-                hint = NAME_TO_HINT.get(window_type)
-                if hint:
-                    log.debug("setting window type to %s - %s", window_type, hint)
-                    self.set_type_hint(hint)
-                    break
-
-    def refresh_window(self):
-        log.debug("Automatic refresh for id %s", self._id)
-        self._client.send_refresh(self._id)
-
-    def refresh_all_windows(self):
-        #this method is only here because we may want to fire it
-        #from a --key-shortcut action and the event is delivered to
-        #the "ClientWindow"
-        self._client.send_refresh_all()
-
-    def draw_region(self, x, y, width, height, coding, img_data, rowstride):
-        self._backing.draw_region(x, y, width, height, coding, img_data, rowstride)
-        queue_draw(self, x, y, width, height)
-        if self._refresh_requested:
-            self._refresh_requested = False
-        else:
-            if self._refresh_timer:
-                gobject.source_remove(self._refresh_timer)
-                self._refresh_timer = None
-            if self._client.auto_refresh_delay and coding == "jpeg":
-                self._refresh_timer = gobject.timeout_add(int(1000 * self._client.auto_refresh_delay), self.refresh_window)
-
-    """ gtk3 """
-    def do_draw(self, context):
-        log.debug("do_draw(%s)", context)
-        if self.get_mapped():
-            self._backing.cairo_draw(context, 0, 0)
-
-    """ gtk2 """
-    def do_expose_event(self, event):
-        log.debug("do_expose_event(%s) area=%s", event, event.area)
-        if not (self.flags() & gtk.MAPPED):
-            return
-        x,y,_,_ = event.area
-        context = self.window.cairo_create()
-        context.rectangle(event.area)
-        context.clip()
-        self._backing.cairo_draw(context, x, y)
-
-    def do_map_event(self, event):
-        log("Got map event")
-        gtk.Window.do_map_event(self, event)
-        if not self._override_redirect:
-            x, y, w, h = get_window_geometry(self)
-            self._client.send(["map-window", self._id, x, y, w, h])
-            self._pos = (x, y)
-            self._size = (w, h)
-        self._been_mapped = True
-        gobject.idle_add(self._focus_change)
-
-    def do_configure_event(self, event):
-        log("Got configure event")
-        gtk.Window.do_configure_event(self, event)
-        if not self._override_redirect:
-            x, y, w, h = get_window_geometry(self)
-            ox, oy = self._pos
-            dx, dy = x-ox, y-oy
-            self._pos = (x, y)
-            if self._client.window_configure:
-                #if we support configure-window, send that first
-                self._client.send(["configure-window", self._id, x, y, w, h])
-            if dx!=0 or dy!=0:
-                #window has moved
-                if not self._client.window_configure:
-                    #if we don't handle the move via configure:
-                    self._client.send(["move-window", self._id, x, y])
-                #move any OR window with their parent:
-                for window in self._override_redirect_windows:
-                    x, y = window.get_position()
-                    window.move(x+dx, y+dy)
-            if (w, h) != self._size:
-                self._size = (w, h)
-                self.new_backing(w, h)
-                if not self._client.window_configure:
-                    self._client.send(["resize-window", self._id, w, h])
-
-    def move_resize(self, x, y, w, h):
-        assert self._override_redirect
-        self.window.move_resize(x, y, w, h)
-        self.new_backing(w, h)
-
-    def destroy(self):
-        self._unfocus()
-        gtk.Window.destroy(self)
-        self._backing.close()
-
-    def _unfocus(self):
-        if self._client._focused==self._id:
-            self._client.update_focus(self._id, False)
-
-    def do_unmap_event(self, event):
-        self._unfocus()
-        if not self._override_redirect:
-            self._client.send(["unmap-window", self._id])
-
-    def do_delete_event(self, event):
-        self._client.send(["close-window", self._id])
-        return True
-
-    def quit(self):
-        self._client.quit()
-
-    def void(self):
-        pass
-
-    def do_key_press_event(self, event):
-        self._client.handle_key_action(event, self, True)
-
-    def do_key_release_event(self, event):
-        self._client.handle_key_action(event, self, False)
-
-    def _pointer_modifiers(self, event):
-        pointer = (int(event.x_root), int(event.y_root))
-        modifiers = self._client.mask_to_names(event.state)
-        return pointer, modifiers
-
-    def do_motion_notify_event(self, event):
-        if self._client.readonly:
-            return
-        (pointer, modifiers) = self._pointer_modifiers(event)
-        self._client.send_mouse_position(["pointer-position", self._id,
-                                          pointer, modifiers])
-
-    def _button_action(self, button, event, depressed):
-        if self._client.readonly:
-            return
-        (pointer, modifiers) = self._pointer_modifiers(event)
-        self._client.send_positional(["button-action", self._id,
-                                      button, depressed,
-                                      pointer, modifiers])
-
-    def do_button_press_event(self, event):
-        if self._client.readonly:
-            return
-        self._button_action(event.button, event, True)
-
-    def do_button_release_event(self, event):
-        if self._client.readonly:
-            return
-        self._button_action(event.button, event, False)
-
-    def do_scroll_event(self, event):
-        if self._client.readonly:
-            return
-        self._button_action(SCROLL_MAP[event.direction], event, True)
-        self._button_action(SCROLL_MAP[event.direction], event, False)
-
-    def _focus_change(self, *args):
-        log("_focus_change(%s)", args)
-        if self._been_mapped:
-            self._client.update_focus(self._id, self.get_property("has-toplevel-focus"))
-
-gobject.type_register(ClientWindow)
 
 
 class XpraClient(XpraClientBase):
     __gsignals__ = {
         "clipboard-toggled": n_arg_signal(0),
+        "keyboard-sync-toggled": n_arg_signal(0),
         }
 
     def __init__(self, conn, opts):
@@ -478,13 +98,13 @@ class XpraClient(XpraClientBase):
         self.server_start_time = -1
         self.server_platform = ""
         self.server_actual_desktop_size = None
-        self.server_desktop_size = None
         self.server_randr = False
         self.pixel_counter = maxdeque(maxlen=100)
         self.server_latency = maxdeque(maxlen=100)
         self.server_load = None
         self.client_latency = maxdeque(maxlen=100)
         self.toggle_cursors_bell_notify = False
+        self.toggle_keyboard_sync = False
         self.bell_enabled = True
         self.cursors_enabled = True
         self.notifications_enabled = True
@@ -494,6 +114,7 @@ class XpraClient(XpraClientBase):
         self.mmap_token = None
         self.mmap_file = None
         self.mmap_size = 0
+        self.last_ping_echoed_time = 0
 
         self._client_extras = ClientExtras(self, opts)
         self.clipboard_enabled = not self.readonly and opts.clipboard and self._client_extras.supports_clipboard()
@@ -583,6 +204,8 @@ class XpraClient(XpraClientBase):
             gobject.timeout_add(2000, compute_receive_bandwidth, 2000)
         if opts.send_pings:
             gobject.timeout_add(1000, self.send_ping)
+        else:
+            gobject.timeout_add(20*1000, self.send_ping)
 
     def init_packet_handlers(self):
         XpraClientBase.init_packet_handlers(self)
@@ -590,6 +213,7 @@ class XpraClient(XpraClientBase):
             "hello":                self._process_hello,
             "new-window":           self._process_new_window,
             "new-override-redirect":self._process_new_override_redirect,
+            "window-resized":       self._process_window_resized,
             "draw":                 self._process_draw,
             "cursor":               self._process_cursor,
             "bell":                 self._process_bell,
@@ -600,6 +224,7 @@ class XpraClient(XpraClientBase):
             "window-metadata":      self._process_window_metadata,
             "configure-override-redirect":  self._process_configure_override_redirect,
             "lost-window":          self._process_lost_window,
+            "desktop_size":         self._process_desktop_size,
             # "clipboard-*" packets are handled by a special case below.
             }.items():
             self._packet_handlers[k] = v
@@ -610,8 +235,9 @@ class XpraClient(XpraClientBase):
         return  self.exit_code
 
     def quit(self, exit_code=0):
+        if self.exit_code is None:
+            self.exit_code = exit_code
         self.cleanup()
-        self.exit_code = exit_code
         gobject.timeout_add(50, gtk_main_quit_really)
 
     def cleanup(self):
@@ -849,11 +475,14 @@ class XpraClient(XpraClientBase):
             self._focused = None
 
     def get_current_modifiers(self):
-        return self.mask_to_names(get_modifiers_mask())
+        modifiers_mask = gdk.get_default_root_window().get_pointer()[-1]
+        return self.mask_to_names(modifiers_mask)
 
     def mask_to_names(self, mask):
         mn = mask_to_names(mask, self._modifier_map)
-        names = self._client_extras.current_modifiers(mn)
+        names = mn
+        if self._client_extras:
+            names = self._client_extras.current_modifiers(mn)
         return  names
 
     def send_positional(self, packet):
@@ -885,6 +514,7 @@ class XpraClient(XpraClientBase):
         if self.mmap_file:
             capabilities["mmap_file"] = self.mmap_file
             capabilities["mmap_token"] = self.mmap_token
+        capabilities["randr_notify"] = True
         #these should be turned into options:
         capabilities["cursors"] = True
         capabilities["bell"] = True
@@ -892,11 +522,20 @@ class XpraClient(XpraClientBase):
         return capabilities
 
     def send_ping(self):
-        self.send(["ping", int(1000*time.time())])
+        now_ms = int(1000*time.time())
+        self.send(["ping", now_ms])
+        wait = 15
+        def check_echo_received(*args):
+            if self.last_ping_echoed_time<now_ms:
+                log.error("check_echo_received: we sent a ping to the server %s seconds ago and we have not received its echo!", wait)
+                log.error("    assuming that the connection is dead and disconnecting")
+                self.quit(1)
+        gobject.timeout_add(wait*1000, check_echo_received)
         return True
 
     def _process_ping_echo(self, packet):
-        (echoedtime, l1, l2, l3, cl) = packet[1:6]
+        echoedtime, l1, l2, l3, cl = packet[1:6]
+        self.last_ping_echoed_time = echoedtime
         diff = int(1000*time.time()-echoedtime)
         self.server_latency.append(diff)
         self.server_load = (l1, l2, l3)
@@ -938,25 +577,17 @@ class XpraClient(XpraClientBase):
             glib.set_application_name(self.session_name)
         except ImportError, e:
             log.warn("glib is missing, cannot set the application name, please install glib's python bindings: %s", e)
-        #figure out the maximum actual desktop size and use to
+        #figure out the maximum actual desktop size and use it to
         #calculate the maximum size of a packet (a full screen update packet)
-        root_w, root_h = get_root_size()
         self.server_actual_desktop_size = capabilities.get("actual_desktop_size")
-        maxw, maxh = root_w, root_h
-        try:
-            server_w, server_h = self.server_actual_desktop_size
-            maxw = max(root_w, server_w)
-            maxh = max(root_h, server_h)
-        except:
-            pass
-        assert maxw>0 and maxh>0 and maxw<32768 and maxh<32768, "problems calculating maximum desktop size: %sx%s" % (maxw, maxh)
-        #full screen at 32bits times 4 for safety
-        self._protocol.max_packet_size = maxw*maxh*4*4
-        self._protocol.raw_packets = bool(capabilities.get("raw_packets", False))
-        log("set maximum packet size to %s", self._protocol.max_packet_size)
-        self.server_desktop_size = capabilities.get("desktop_size")
-        assert self.server_desktop_size
-        avail_w, avail_h = self.server_desktop_size
+        log("server actual desktop size=%s", self.server_actual_desktop_size)
+        self.set_max_packet_size()
+        self.server_max_desktop_size = capabilities.get("max_desktop_size")
+        server_desktop_size = capabilities.get("desktop_size")
+        log("server desktop size=%s", server_desktop_size)
+        assert server_desktop_size
+        avail_w, avail_h = server_desktop_size
+        root_w, root_h = get_root_size()
         if avail_w<root_w or avail_h<root_h:
             log.warn("Server's virtual screen is too small -- "
                      "(server: %sx%s vs. client: %sx%s)\n"
@@ -989,7 +620,8 @@ class XpraClient(XpraClientBase):
         self.send_deflate_level()
         self.server_start_time = capabilities.get("start_time", -1)
         self.server_platform = capabilities.get("platform")
-        self.toggle_cursors_bell_notify = capabilities.get("toggle_cursors_bell_notify")
+        self.toggle_cursors_bell_notify = capabilities.get("toggle_cursors_bell_notify", False)
+        self.toggle_keyboard_sync = capabilities.get("toggle_keyboard_sync", False)
         #ui may want to know this is now set:
         self.emit("clipboard-toggled")
         self.key_repeat_delay, self.key_repeat_interval = capabilities.get("key_repeat", (-1,-1))
@@ -997,6 +629,8 @@ class XpraClient(XpraClientBase):
         if clipboard_server_support:
             #from now on, we will send a message to the server whenever the clipboard flag changes:
             self.connect("clipboard-toggled", self.send_clipboard_enabled_status)
+        if self.toggle_keyboard_sync:
+            self.connect("keyboard-sync-toggled", self.send_keyboard_sync_enabled_status)
 
     def send_notify_enabled(self):
         if self.toggle_cursors_bell_notify:
@@ -1016,6 +650,9 @@ class XpraClient(XpraClientBase):
     def send_clipboard_enabled_status(self, *args):
         self.send(["set-clipboard-enabled", self.clipboard_enabled])
 
+    def send_keyboard_sync_enabled_status(self, *args):
+        self.send(["set-keyboard-sync-enabled", self.keyboard_sync])
+
     def set_encoding(self, encoding):
         assert encoding in ENCODINGS
         server_encodings = self.server_capabilities.get("encodings", [])
@@ -1027,6 +664,8 @@ class XpraClient(XpraClientBase):
         root_w, root_h = get_root_size()
         log.debug("sending updated screen size to server: %sx%s", root_w, root_h)
         self.send(["desktop_size", root_w, root_h])
+        #update the max packet size (may have gone up):
+        self.set_max_packet_size()
 
     def _process_new_common(self, packet, override_redirect):
         (wid, x, y, w, h, metadata) = packet[1:7]
@@ -1034,7 +673,7 @@ class XpraClient(XpraClientBase):
             log.error("window dimensions are wrong: %sx%s", w, h)
             w = 10
             h = 5
-        window = ClientWindow(self, wid, x, y, w, h, metadata, override_redirect)
+        window = ClientWindowClass(self, wid, x, y, w, h, metadata, override_redirect)
         self._id_to_window[wid] = window
         self._window_to_id[window] = wid
         window.show_all()
@@ -1045,17 +684,24 @@ class XpraClient(XpraClientBase):
     def _process_new_override_redirect(self, packet):
         self._process_new_common(packet, True)
 
+    def _process_window_resized(self, packet):
+        (wid, w, h) = packet[1:4]
+        window = self._id_to_window.get(wid)
+        log("_process_window_resized resizing window %s (id=%s) to %s", window, wid, (w,h))
+        if window:
+            window.resize(w, h)
+
     def _process_draw(self, packet):
         (wid, x, y, width, height, coding, data, packet_sequence, rowstride) = packet[1:10]
         window = self._id_to_window.get(wid)
+        decode_time = 0
         if window:
             start = time.time()
-            window.draw_region(x, y, width, height, coding, data, rowstride)
-            end = time.time()
-            self.pixel_counter.append((end, width*height))
-            decode_time = int(end*1000*1000-start*1000*1000)
+            if window.draw_region(x, y, width, height, coding, data, rowstride):
+                end = time.time()
+                self.pixel_counter.append((end, width*height))
+                decode_time = int(end*1000*1000-start*1000*1000)
         else:
-            decode_time = 0
             #window is gone
             if coding=="mmap":
                 #we need to ack the data to free the space!
@@ -1122,6 +768,26 @@ class XpraClient(XpraClientBase):
             log.debug("last window gone, clearing key repeat")
             self.clear_repeat()
 
+    def _process_desktop_size(self, packet):
+        root_w, root_h, max_w, max_h = packet[1:5]
+        log("server has resized the desktop to: %sx%s (max %sx%s)", root_w, root_h, max_w, max_h)
+        self.server_max_desktop_size = max_w, max_h
+        self.server_actual_desktop_size = root_w, root_h
+
+    def set_max_packet_size(self):
+        root_w, root_h = get_root_size()
+        maxw, maxh = root_w, root_h
+        try:
+            server_w, server_h = self.server_actual_desktop_size
+            maxw = max(root_w, server_w)
+            maxh = max(root_h, server_h)
+        except:
+            pass
+        assert maxw>0 and maxh>0 and maxw<32768 and maxh<32768, "problems calculating maximum desktop size: %sx%s" % (maxw, maxh)
+        #full screen at 32bits times 4 for safety
+        self._protocol.max_packet_size = maxw*maxh*4*4
+        log("set maximum packet size to %s", self._protocol.max_packet_size)
+
     def process_packet(self, proto, packet):
         packet_type = str(packet[0])
         if packet_type.startswith("clipboard-"):
