@@ -20,10 +20,11 @@ import datetime
 from xpra.platform import XPRA_LOCAL_SERVERS_SUPPORTED
 from xpra.scripts.main import ENCODINGS
 from xpra.keys import get_gtk_keymap
-from wimpiggy.util import gtk_main_quit_really
 from wimpiggy.log import Logger
 log = Logger()
 
+#compression is fine with default value (3), no need to clutter the UI
+SHOW_COMPRESSION_MENU = False
 
 #utility method to ensure there is always only one CheckMenuItem
 #selected in a submenu:
@@ -121,7 +122,7 @@ class ClientExtrasBase(object):
                 log.error("failed to set window icon %s: %s, continuing", window_icon, e)
 
     def quit(self, *args):
-        gtk_main_quit_really()
+        self.client.quit(0)
 
     def exit(self):
         self.close_about()
@@ -217,20 +218,26 @@ class ClientExtrasBase(object):
         if pixbuf:
             dialog.set_logo(pixbuf)
         dialog.set_program_name("Xpra")
-        comments = ''
-        try:
-            from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, REVISION, LOCAL_MODIFICATIONS
-            comments += "Built on %s by %s. %s" % (BUILT_ON, BUILT_BY, BUILD_DATE)
-            if LOCAL_MODIFICATIONS==0:
-                comments += "\n(svn revision %s)" % REVISION
-            else:
-                comments += "\n(svn revision %s with %s local changes)" % (REVISION, LOCAL_MODIFICATIONS)
-        except Exception, e:
-            log.error("could not find the build information: %s", e)
-        dialog.set_comments(comments)
+        dialog.set_comments("\n".join(self.get_build_info()))
         dialog.connect("response", self.close_about)
         self.about_dialog = dialog
         dialog.show()
+        dialog.present()
+
+    def get_build_info(self):
+        info = []
+        try:
+            from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, REVISION, LOCAL_MODIFICATIONS
+            info.append("Built on %s by %s" % (BUILT_ON, BUILT_BY))
+            if BUILD_DATE:
+                info.append(BUILD_DATE)
+            if LOCAL_MODIFICATIONS==0:
+                info.append("revision %s" % REVISION)
+            else:
+                info.append("revision %s with %s local changes" % (REVISION, LOCAL_MODIFICATIONS))
+        except Exception, e:
+            log.error("could not find the build information: %s", e)
+        return info
 
     def close_about(self, *args):
         try:
@@ -275,41 +282,68 @@ class ClientExtrasBase(object):
             w_al.add(widget)
             table.attach(w_al, 1, 2, row, row + 1, xpadding=10)
             return row + 1
+        def label(text="", tooltip=None):
+            l = gtk.Label(text)
+            if tooltip:
+                l.set_tooltip_text(tooltip)
+            return l
+        def new_row(row, row_label_text, value_label, tooltip_text=None):
+            row_label = label(row_label_text, tooltip_text)
+            if tooltip_text:
+                value_label.set_tooltip_text(tooltip_text)
+            return add_row(row, row_label, value_label)
 
         # now add some rows with info:
         row = 0
+        from xpra.__init__ import __version__
+        row = new_row(row, "Xpra version", label(__version__))
+        row = new_row(row, "Xpra build", label("\n".join(self.get_build_info())))
+        self.server_version_label = label()
+        row = new_row(row, "Server Version", self.server_version_label)
         if is_gtk3():
-            row = add_row(row, gtk.Label("PyGobject version"), gtk.Label(gobject._version))
-            row = add_row(row, gtk.Label("GTK version"), gtk.Label(gtk._version))
-            row = add_row(row, gtk.Label("GDK version"), gtk.Label(gdk._version))
+            row = new_row(row, "PyGobject version", label(gobject._version))
+            row = new_row(row, "GTK version", label(gtk._version))
+            row = new_row(row, "GDK version", label(gdk._version))
         else:
-            row = add_row(row, gtk.Label("PyGTK version"), gtk.Label(".".join([str(x) for x in gtk.pygtk_version])))
-            row = add_row(row, gtk.Label("GTK version"), gtk.Label(".".join([str(x) for x in gtk.gtk_version])))
+            def make_version_str(version):
+                return  ".".join([str(x) for x in version])
+            def make_version_info(prop_name):
+                info = "unknown"
+                if hasattr(gtk, prop_name):
+                    info = make_version_str(getattr(gtk, prop_name))
+                server_version = self.client.server_capabilities.get(prop_name)
+                if server_version:
+                    info += " (server: %s)" % make_version_str(server_version)
+                return info
+            row = new_row(row, "PyGTK version", label(make_version_info("pygtk_version")))
+            row = new_row(row, "GTK version", label(make_version_info("gtk_version")))
 
-        self.server_version_label = gtk.Label()
-        row = add_row(row, gtk.Label("Server Version"), self.server_version_label)
         if self.client.server_platform:
-            row = add_row(row, gtk.Label("Server Platform"), gtk.Label(self.client.server_platform))
-        self.server_randr_label = gtk.Label()
-        row = add_row(row, gtk.Label("Server RandR Support"), self.server_randr_label)
-        self.server_load_label = gtk.Label()
-        row = add_row(row, gtk.Label("Server Load"), self.server_load_label)
-        self.server_latency_label = gtk.Label()
-        row = add_row(row, gtk.Label("Server Latency"), self.server_latency_label)
-        self.client_latency_label = gtk.Label()
-        row = add_row(row, gtk.Label("Client Latency"), self.client_latency_label)
-        self.session_started_label = gtk.Label()
-        row = add_row(row, gtk.Label("Session Started"), self.session_started_label)
-        self.session_connected_label = gtk.Label()
-        row = add_row(row, gtk.Label("Session Connected"), self.session_connected_label)
-        self.windows_managed_label = gtk.Label()
-        row = add_row(row, gtk.Label("Windows Managed"), self.windows_managed_label)
-        self.regions_sizes_label = gtk.Label()
-        row = add_row(row, gtk.Label("Pixels/region (min/avg/max)"), self.regions_sizes_label)
-        self.regions_per_second_label = gtk.Label()
-        row = add_row(row, gtk.Label("Regions/s"), self.regions_per_second_label)
-        self.pixels_per_second_label = gtk.Label()
-        row = add_row(row, gtk.Label("Pixels/s"), self.pixels_per_second_label)
+            row = new_row(row, "Server Platform", label(self.client.server_platform))
+        self.server_randr_label = label()
+        row = new_row(row, "Server RandR Support", self.server_randr_label)
+        self.server_load_label = label()
+        row = new_row(row, "Server Load", self.server_load_label, "Average over 1, 5 and 15 minutes")
+        self.server_latency_label = label()
+        row = new_row(row, "Server Latency", self.server_latency_label, "last value and average")
+        self.client_latency_label = label()
+        row = new_row(row, "Client Latency", self.client_latency_label, "last value and average")
+        self.session_started_label = label()
+        row = new_row(row, "Session Started", self.session_started_label)
+        self.session_connected_label = label()
+        row = new_row(row, "Session Connected", self.session_connected_label)
+        self.windows_managed_label = label()
+        row = new_row(row, "Windows Managed", self.windows_managed_label,
+                      "The number of windows forwarded, some may just be temporary widgets (usually transient ones)")
+        self.regions_sizes_label = label()
+        row = new_row(row, "Pixels/region", self.regions_sizes_label,
+                      "The number of pixels updated at a time: min/avg/max")
+        self.regions_per_second_label = label()
+        row = new_row(row, "Regions/s", self.regions_per_second_label,
+                      "The number of screen updates per second")
+        self.pixels_per_second_label = label()
+        row = new_row(row, "Pixels/s", self.pixels_per_second_label,
+                      "The number of pixels updated per second")
 
         def populate_table(*args):
             if not self.session_info_window:
@@ -327,6 +361,8 @@ class ClientExtrasBase(object):
             if self.client.server_actual_desktop_size:
                 w,h = self.client.server_actual_desktop_size
                 size_info = " - %sx%s" % (w,h)
+                if self.client.server_randr and self.client.server_max_desktop_size:
+                    size_info += " (max %s)" % ("x".join([str(x) for x in self.client.server_max_desktop_size]))
             if self.client.server_randr:
                 self.server_randr_label.set_text("Yes%s" % size_info)
             else:
@@ -424,6 +460,7 @@ class ClientExtrasBase(object):
         self.session_info_window = window
         populate_table()
         window.show_all()
+        window.present()
 
     def close_session_info(self, *args):
         try:
@@ -639,6 +676,27 @@ class ClientExtrasBase(object):
         self.client.connect("handshake-complete", set_clipboard_menuitem)
         return self.clipboard_menuitem
 
+    def make_keyboardsyncmenuitem(self):
+        def set_keyboard_sync_tooltip():
+            if not self.client.toggle_keyboard_sync:
+                set_tooltip_text(self.keyboard_sync_menuitem, "This server does not support changes to keyboard synchronization")
+            elif self.client.keyboard_sync:
+                set_tooltip_text(self.keyboard_sync_menuitem, "Disable keyboard synchronization (prevents spurious key repeats on high latency connections)")
+            else:
+                set_tooltip_text(self.keyboard_sync_menuitem, "Enable keyboard state synchronization")
+        def keyboard_sync_toggled(*args):
+            self.client.keyboard_sync = self.keyboard_sync_menuitem.get_active()
+            log.debug("keyboard_sync_toggled(%s) keyboard_sync=%s", args, self.client.keyboard_sync)
+            set_keyboard_sync_tooltip()
+            self.client.emit("keyboard-sync-toggled")
+        self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
+        def set_keyboard_sync_menuitem(*args):
+            self.keyboard_sync_menuitem.set_active(self.client.keyboard_sync)
+            self.keyboard_sync_menuitem.set_sensitive(self.client.toggle_keyboard_sync)
+            set_keyboard_sync_tooltip()
+        self.client.connect("handshake-complete", set_keyboard_sync_menuitem)
+        return self.keyboard_sync_menuitem
+
     def make_encodingsmenuitem(self):
         encodings = self.menuitem("Encoding", "encoding.png", "Choose picture data encoding", None)
         self.encodings_submenu = gtk.Menu()
@@ -669,6 +727,41 @@ class ClientExtrasBase(object):
         self.client.connect("handshake-complete", set_encodingsmenuitem)
         return encodings
 
+    def make_jpegsubmenu(self):
+        self.jpeg_quality = self.menuitem("JPEG Quality", "slider.png", "Change JPEG quality setting", None)
+        self.jpeg_submenu = gtk.Menu()
+        self.jpeg_quality.set_submenu(self.jpeg_submenu)
+        self.popup_menu_workaround(self.jpeg_submenu)
+        jpeg_options = [10, 50, 80, 95]
+        if self.client.jpegquality>0 and self.client.jpegquality not in jpeg_options:
+            """ add the current value to the list of options """
+            i = 0
+            for x in jpeg_options:
+                if self.client.jpegquality<x:
+                    jpeg_options.insert(i, self.client.jpegquality)
+                    break
+                i += 1
+        def set_jpeg_quality(item):
+            item = ensure_item_selected(self.jpeg_submenu, item)
+            q = int(item.get_label().replace("%", ""))
+            if q!=self.client.jpegquality:
+                log.debug("setting jpeg quality to %s", q)
+                self.client.send_jpeg_quality(q)
+        for q in jpeg_options:
+            qi = CheckMenuItem("%s%%" % q)
+            qi.set_draw_as_radio(True)
+            qi.set_active(q==self.client.jpegquality)
+            qi.connect('activate', set_jpeg_quality)
+            self.jpeg_submenu.append(qi)
+        self.jpeg_submenu.show_all()
+        self.client.connect("handshake-complete", self.set_jpegmenu)
+        return self.jpeg_quality
+
+    def set_jpegmenu(self, *args):
+        if self.jpeg_quality:
+            self.jpeg_quality.set_sensitive("jpeg"==self.client.encoding)
+            self.updated_menus()
+
     def make_layoutsmenuitem(self):
         keyboard = self.menuitem("Keyboard", "keyboard.png", "Select your keyboard layout", None)
         self.layout_submenu = gtk.Menu()
@@ -734,41 +827,6 @@ class ClientExtrasBase(object):
         self.client.connect("handshake-complete", set_selected_layout)
         return keyboard
 
-    def make_jpegsubmenu(self):
-        self.jpeg_quality = self.menuitem("JPEG Quality", "slider.png", "Change JPEG quality setting", None)
-        self.jpeg_submenu = gtk.Menu()
-        self.jpeg_quality.set_submenu(self.jpeg_submenu)
-        self.popup_menu_workaround(self.jpeg_submenu)
-        jpeg_options = [10, 50, 80, 95]
-        if self.client.jpegquality>0 and self.client.jpegquality not in jpeg_options:
-            """ add the current value to the list of options """
-            i = 0
-            for x in jpeg_options:
-                if self.client.jpegquality<x:
-                    jpeg_options.insert(i, self.client.jpegquality)
-                    break
-                i += 1
-        def set_jpeg_quality(item):
-            item = ensure_item_selected(self.jpeg_submenu, item)
-            q = int(item.get_label().replace("%", ""))
-            if q!=self.client.jpegquality:
-                log.debug("setting jpeg quality to %s", q)
-                self.client.send_jpeg_quality(q)
-        for q in jpeg_options:
-            qi = CheckMenuItem("%s%%" % q)
-            qi.set_draw_as_radio(True)
-            qi.set_active(q==self.client.jpegquality)
-            qi.connect('activate', set_jpeg_quality)
-            self.jpeg_submenu.append(qi)
-        self.jpeg_submenu.show_all()
-        self.client.connect("handshake-complete", self.set_jpegmenu)
-        return self.jpeg_quality
-
-    def set_jpegmenu(self, *args):
-        if self.jpeg_quality:
-            self.jpeg_quality.set_sensitive("jpeg"==self.client.encoding)
-            self.updated_menus()
-
     def make_compressionmenu(self):
         self.compression = self.menuitem("Compression", "compressed.png", "Network packet compression", None)
         self.compression_submenu = gtk.Menu()
@@ -841,9 +899,11 @@ class ClientExtrasBase(object):
         else:
             self.jpeg_quality = None
             self.jpeg_submenu = None
-        menu.append(self.make_compressionmenu())
+        if SHOW_COMPRESSION_MENU:
+            menu.append(self.make_compressionmenu())
         if not self.client.readonly:
             menu.append(self.make_layoutsmenuitem())
+        menu.append(self.make_keyboardsyncmenuitem())
         menu.append(self.make_refreshmenuitem())
         menu.append(self.make_raisewindowsmenuitem())
         #menu.append(item("Options", "configure", None, self.options))
